#*******************************************************************************
# Copyright (C) 2008 Novell, Inc. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of Novell, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
#*****************************************************************************/
"""Python Provider for OMC_AssociatedPowerManagementService

Instruments the CIM class OMC_AssociatedPowerManagementService

"""

import os
import pywbem
from pywbem.cim_provider2 import CIMProvider2
from datetime import datetime, timedelta

import OMC_PowerManagementService
get_service_path = OMC_PowerManagementService.get_path 
service_id = OMC_PowerManagementService.inst_id
script_file = OMC_PowerManagementService.script_file
script_pid = OMC_PowerManagementService.script_pid

from socket import getfqdn

from OMC_UnixProcess import LinuxProcess

def get_shutdown_time():
    try:
        fo = open(script_pid, 'r')
    except IOError:
        return None
    spid = fo.read().strip()
    fo.close()
    lp = LinuxProcess(spid)
    cmdline = lp.get_command_line()
    if len(cmdline) < 3:
        return None
    if cmdline[1] == script_file:
        del cmdline[0]
    if cmdline[0] != script_file:
        return None
    if len(cmdline) < 3:
        return None
    requested_state = pywbem.Uint16(cmdline[2])
    gotime = pywbem.CIMDateTime(cmdline[1])
    return (requested_state, gotime)



class OMC_AssociatedPowerManagementService(CIMProvider2):
    """Instrument the CIM class OMC_AssociatedPowerManagementService 

    The association between a Managed System Element and its power
    management service.
    
    """

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))

    def get_instance(self, env, model):
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        

        # TODO fetch system resource matching the following keys:
        #   model['UserOfService']
        #   model['ServiceProvided']

        #model['OtherPowerState'] = '' # TODO 
        #model['OtherRequestedPowerState'] = '' # TODO 
        rv = get_shutdown_time()
        if rv is None:
            shutdowntime = None
        else:
            shutdowntime = rv[1]
        model['PowerOnTime'] = pywbem.CIMProperty('PowerOnTime', 
                value=shutdowntime, type='datetime')
        if shutdowntime is not None:
            model['RequestedPowerState'] = rv[0]
        else:
            model['RequestedPowerState'] = self.Values.RequestedPowerState.Not_Applicable 
        model['PowerState'] = self.Values.PowerState.On
        return model

    def enum_instances(self, env, model, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
                
        # Prime model.path with knowledge of the keys, so key values on
        # the CIMInstanceName (model.path) will automatically be set when
        # we set property values on the model. 
        model.path.update({u'UserOfService': None, u'ServiceProvided': None})
        service = pywbem.CIMInstanceName(
                classname='OMC_PowerManagementService', 
                namespace=model.path.namespace)
        get_service_path(service)

        cs = pywbem.CIMInstanceName(
                classname='OMC_ComputerSystem', 
                namespace=model.path.namespace,
                keybindings={'CreationClassName':'OMC_ComputerSystem',
                            'Name':getfqdn()})

        model['UserOfService'] = cs
        model['ServiceProvided'] = service
        
        if keys_only:
            yield model
        else:
            try:
                yield self.get_instance(env, model)
            except pywbem.CIMError, (num, msg):
                if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                               pywbem.CIM_ERR_ACCESS_DENIED):
                    raise

    def set_instance(self, env, instance, modify_existing):

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):

        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.references()' \
                % self.__class__.__name__)
        ch = env.get_cimom_handle()

        # If you want to get references for free, implemented in terms 
        # of enum_instances, just leave the code below unaltered.
        if ch.is_subclass(object_name.namespace, 
                          sub=object_name.classname,
                          super='OMC_UnitaryComputerSystem') or \
                ch.is_subclass(object_name.namespace,
                               sub=object_name.classname,
                               super='OMC_PowerManagementService'):
            return self.simple_refs(env, object_name, model,
                          result_class_name, role, result_role, keys_only)
                          

    class Values(object):
        class PowerState(object):
            Other = pywbem.Uint16(1)
            On = pywbem.Uint16(2)
            Sleep___Light = pywbem.Uint16(3)
            Sleep__Deep = pywbem.Uint16(4)
            Power_Cycle__Off___Soft_ = pywbem.Uint16(5)
            Off___Hard = pywbem.Uint16(6)
            Hibernate__Off___Soft_ = pywbem.Uint16(7)
            Off___Soft = pywbem.Uint16(8)
            Power_Cycle__Off_Hard_ = pywbem.Uint16(9)
            Master_Bus_Reset = pywbem.Uint16(10)
            Diagnostic_Interrupt__NMI_ = pywbem.Uint16(11)
            Off___Soft_Graceful = pywbem.Uint16(12)
            Off___Hard_Graceful = pywbem.Uint16(13)
            Master_Bus_Reset_Graceful = pywbem.Uint16(14)
            Power_Cycle__Off___Soft_Graceful_ = pywbem.Uint16(15)
            Power_Cycle__Off___Hard_Graceful_ = pywbem.Uint16(16)
            # DMTF_Reserved = ..
            # Vendor_Specific = 0x7FFF..0xFFFF

        class RequestedPowerState(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            On = pywbem.Uint16(2)
            Sleep___Light = pywbem.Uint16(3)
            Sleep___Deep = pywbem.Uint16(4)
            Power_Cycle__Off___Soft_ = pywbem.Uint16(5)
            Off___Hard = pywbem.Uint16(6)
            Hibernate__Off___Soft_ = pywbem.Uint16(7)
            Off___Soft = pywbem.Uint16(8)
            Power_Cycle__Off_Hard_ = pywbem.Uint16(9)
            Master_Bus_Reset = pywbem.Uint16(10)
            Diagnostic_Interrupt__NMI_ = pywbem.Uint16(11)
            Not_Applicable = pywbem.Uint16(12)
            Off___Soft_Graceful = pywbem.Uint16(13)
            Off___Hard_Graceful = pywbem.Uint16(14)
            Master_Bus_Reset_Graceful = pywbem.Uint16(15)
            Power_Cycle__Off___Soft_Graceful_ = pywbem.Uint16(16)
            Power_Cycle__Off___Hard_Graceful_ = pywbem.Uint16(17)
            # DMTF_Reserved = ..
            # Vendor_Specific = 0x7FFF..0xFFFF

## end of class OMC_AssociatedPowerManagementServiceProvider

def get_providers(env): 
    omc_associatedpowermanagementservice_prov = OMC_AssociatedPowerManagementService(env)  
    return {'OMC_AssociatedPowerManagementService': omc_associatedpowermanagementservice_prov} 



if __name__ == '__main__':
    print get_shutdown_time()




