#*******************************************************************************
# Copyright (C) 2008 Novell, Inc. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of Novell, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
#*****************************************************************************/
"""Python Provider for OMC_PowerManagementCapabilities

Instruments the CIM class OMC_PowerManagementCapabilities

"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2

from OMC_PowerManagementService import get_supported_states

inst_id = 'SUSE:PowerManagementCapabilities'

def get_path(mapping):
    mapping['InstanceID'] = inst_id
    return mapping


class OMC_PowerManagementCapabilitiesProvider(CIMProvider2):
    """Instrument the CIM class OMC_PowerManagementCapabilities 

    A class derived from Capabilities that describes the power management
    aspects of an element (typically a system or device). The power
    management capabilities of the element are decoupled from a
    PowerManagementService, because a single service could apply to
    multiple elements, each with specific capabilities.
    
    """

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))

    def get_instance(self, env, model):
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        

        if model['InstanceID'] != inst_id:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)

        #model['Caption'] = '' # TODO 
        #model['Description'] = '' # TODO 
        model['ElementName'] = 'Power Management Capabilities'
        #model['Generation'] = pywbem.Uint64() # TODO 
        #model['OtherPowerCapabilitiesDescriptions'] = ['',] # TODO 
        #model['OtherPowerChangeCapabilities'] = '' # TODO 
        #model['PowerCapabilities'] = [self.Values.PowerCapabilities.<VAL>,] # TODO 
        states = get_supported_states()

        caps = set([capvals.Power_State_Settable])

        for state in states.keys():
            if state in cap_map:
                caps.update(cap_map[state])


        model['PowerChangeCapabilities'] = list(caps)
        model['PowerStatesSupported'] = states.keys()
        return model

    def enum_instances(self, env, model, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
                
        # Prime model.path with knowledge of the keys, so key values on
        # the CIMInstanceName (model.path) will automatically be set when
        # we set property values on the model. 
        model.path.update({u'InstanceID': None})
        
        get_path(model)
        if keys_only:
            yield model
        else:
            try:
                yield self.get_instance(env, model)
            except pywbem.CIMError, (num, msg):
                if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                               pywbem.CIM_ERR_ACCESS_DENIED):
                    raise

    def set_instance(self, env, instance, modify_existing):

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
    def cim_method_creategoalsettings(self, env, object_name,
                                      param_supportedgoalsettings,
                                      param_templategoalsettings):
        """Implements OMC_PowerManagementCapabilities.CreateGoalSettings()

        Method to create a set of supported SettingData elements, from two
        sets of SettingData elements, provided by the caller. \nCreateGoal
        should be used when the SettingData instances that represents the
        goal will not persist beyond the execution of the client and where
        those instances are not intended to be shared with other,
        non-cooperating clients. \nBoth TemplateGoalSettings and
        SupportedGoalSettings are represented as strings containing
        EmbeddedInstances of a CIM_SettingData subclass. These embedded
        instances do not exist in the infrastructure supporting this
        method but are maintained by the caller/client. \nThis method
        should return CIM_Error(s) representing that a single named
        property of a setting (or other) parameter (either reference or
        embedded object) has an invalid value or that an invalid
        combination of named properties of a setting (or other) parameter
        (either reference or embedded object) has been requested. \nIf the
        input TemplateGoalSettings is NULL or the empty string, this
        method returns a default SettingData element that is supported by
        this Capabilities element. \nIf the TemplateGoalSettings specifies
        values that cannot be supported, this method shall return an
        appropriate CIM_Error and should return a best match for a
        SupportedGoalSettings. \nThe client proposes a goal using the
        TemplateGoalSettings parameter and gets back Success if the
        TemplateGoalSettings is exactly supportable. It gets back
        "Alternative Proposed" if the output SupportedGoalSettings
        represents a supported alternative. This alternative should be a
        best match, as defined by the implementation. \nIf the
        implementation is conformant to a RegisteredProfile, then that
        profile may specify the algorithms used to determine best match. A
        client may compare the returned value of each property against the
        requested value to determine if it is left unchanged, degraded or
        upgraded. \n\nOtherwise, if the TemplateGoalSettings is not
        applicable an "Invalid Parameter" error is returned. \n\nWhen a
        mutually acceptable SupportedGoalSettings has been achieved, the
        client may use the contained SettingData instances as input to
        methods for creating a new object ormodifying an existing object.
        Also the embedded SettingData instances returned in the
        SupportedGoalSettings may be instantiated via CreateInstance,
        either by a client or as a side-effect of the execution of an
        extrinsic method for which the returned SupportedGoalSettings is
        passed as an embedded instance.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method CreateGoalSettings() 
            should be invoked.
        param_supportedgoalsettings --  The input parameter SupportedGoalSettings (type pywbem.CIMInstance(classname='CIM_SettingData', ...)) 
            SupportedGoalSettings are elements of class CIM_SettingData, or
            a derived class. \nAt most, one instance of each SettingData
            subclass may be supplied. \nAll SettingData instances provided
            by this property are interpreted as a set, relative to this
            Capabilities instance. \n\nTo enable a client to provide
            additional information towards achieving the
            TemplateGoalSettings, an input set of SettingData instances
            may be provided. If not provided, this property shall be set
            to NULL on input.. Note that when provided, what property
            values are changed, and how, is implementation dependent and
            may be the subject of other standards. \nIf provided, the
            input SettingData instances must be ones that the
            implementation is able to support relative to the
            ManagedElement associated via ElementCapabilities. Typically,
            the input SettingData instances are created by a previous
            instantiation of CreateGoalSettings. \nIf the input
            SupportedGoalSettings is not supported by the implementation,
            then an "Invalid Parameter" (5) error is returned by this
            call. In this case, a corresponding CIM_ERROR should also be
            returned. \nOn output, this property is used to return the
            best supported match to the TemplateGoalSettings. \nIf the
            output SupportedGoalSettings matches the input
            SupportedGoalSettings, then the implementation is unable to
            improve further towards meeting the TemplateGoalSettings.
            
        param_templategoalsettings --  The input parameter TemplateGoalSettings (type pywbem.CIMInstance(classname='CIM_SettingData', ...)) 
            If provided, TemplateGoalSettings are elements of class
            CIM_SettingData, or a derived class, that is used as the
            template to be matched. . \nAt most, one instance of each
            SettingData subclass may be supplied. \nAll SettingData
            instances provided by this property are interpreted as a set,
            relative to this Capabilities instance. \nSettingData
            instances that are not relevant to this instance are ignored.
            \nIf not provided, it shall be set to NULL. In that case, a
            SettingData instance representing the default settings of the
            associated ManagedElement is used.
            

        Returns a two-tuple containing the return value (type pywbem.Uint16 self.Values.CreateGoalSettings)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        SupportedGoalSettings -- (type pywbem.CIMInstance(classname='CIM_SettingData', ...)) 
            SupportedGoalSettings are elements of class CIM_SettingData, or
            a derived class. \nAt most, one instance of each SettingData
            subclass may be supplied. \nAll SettingData instances provided
            by this property are interpreted as a set, relative to this
            Capabilities instance. \n\nTo enable a client to provide
            additional information towards achieving the
            TemplateGoalSettings, an input set of SettingData instances
            may be provided. If not provided, this property shall be set
            to NULL on input.. Note that when provided, what property
            values are changed, and how, is implementation dependent and
            may be the subject of other standards. \nIf provided, the
            input SettingData instances must be ones that the
            implementation is able to support relative to the
            ManagedElement associated via ElementCapabilities. Typically,
            the input SettingData instances are created by a previous
            instantiation of CreateGoalSettings. \nIf the input
            SupportedGoalSettings is not supported by the implementation,
            then an "Invalid Parameter" (5) error is returned by this
            call. In this case, a corresponding CIM_ERROR should also be
            returned. \nOn output, this property is used to return the
            best supported match to the TemplateGoalSettings. \nIf the
            output SupportedGoalSettings matches the input
            SupportedGoalSettings, then the implementation is unable to
            improve further towards meeting the TemplateGoalSettings.
            

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, 
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not 
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor 
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_creategoalsettings()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = []
        #out_params+= [pywbem.CIMParameter('supportedgoalsettings', type='string', 
        #                   value=[pywbem.CIMInstance(classname='CIM_SettingData', ...),])] # TODO
        #rval = # TODO (type pywbem.Uint16 self.Values.CreateGoalSettings)
        return (rval, out_params)
        
    class Values(object):
        class PowerCapabilities(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Power_Saving_Modes_Entered_Automatically = pywbem.Uint16(2)
            Power_State_Settable = pywbem.Uint16(3)
            Power_Cycling_Supported = pywbem.Uint16(4)
            Timed_Power_On_Supported = pywbem.Uint16(5)

        class PowerStatesSupported(object):
            Other = pywbem.Uint16(1)
            On = pywbem.Uint16(2)
            Sleep___Light = pywbem.Uint16(3)
            Sleep__Deep = pywbem.Uint16(4)
            Power_Cycle__Off___Soft_ = pywbem.Uint16(5)
            Off___Hard = pywbem.Uint16(6)
            Hibernate__Off___Soft_ = pywbem.Uint16(7)
            Off___Soft = pywbem.Uint16(8)
            Power_Cycle__Off_Hard_ = pywbem.Uint16(9)
            Master_Bus_Reset = pywbem.Uint16(10)
            Diagnostic_Interrupt__NMI_ = pywbem.Uint16(11)
            Off___Soft_Graceful = pywbem.Uint16(12)
            Off___Hard_Graceful = pywbem.Uint16(13)
            Master_Bus_Reset_Graceful = pywbem.Uint16(14)
            Power_Cycle__Off___Soft_Graceful_ = pywbem.Uint16(15)
            Power_Cycle__Off___Hard_Graceful_ = pywbem.Uint16(16)
            # DMTF_Reserved = ..
            # Vendor_Specific = 0x7FFF..0xFFFF

        class CreateGoalSettings(object):
            Success = pywbem.Uint16(0)
            Not_Supported = pywbem.Uint16(1)
            Unknown = pywbem.Uint16(2)
            Timeout = pywbem.Uint16(3)
            Failed = pywbem.Uint16(4)
            Invalid_Parameter = pywbem.Uint16(5)
            Alternative_Proposed = pywbem.Uint16(6)
            # DMTF_Reserved = ..
            # Vendor_Specific = 32768..65535

        class PowerChangeCapabilities(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Power_Saving_Modes_Entered_Automatically = pywbem.Uint16(2)
            Power_State_Settable = pywbem.Uint16(3)
            Power_Cycling_Supported = pywbem.Uint16(4)
            Timed_Power_On_Supported = pywbem.Uint16(5)
            Off_Hard_Power_Cycling_Supported = pywbem.Uint16(6)
            HW_Reset_Supported = pywbem.Uint16(7)
            Graceful_Shutdown_Supported = pywbem.Uint16(8)

## end of class OMC_PowerManagementCapabilitiesProvider

def get_providers(env): 
    omc_powermanagementcapabilities_prov = OMC_PowerManagementCapabilitiesProvider(env)  
    return {'OMC_PowerManagementCapabilities': omc_powermanagementcapabilities_prov} 

capvals = OMC_PowerManagementCapabilitiesProvider.Values.PowerChangeCapabilities
psvals = OMC_PowerManagementCapabilitiesProvider.Values.PowerStatesSupported
cap_map = {
psvals.Power_Cycle__Off___Soft_:[capvals.Power_Cycling_Supported],
psvals.Power_Cycle__Off_Hard_ :[capvals.Off_Hard_Power_Cycling_Supported],
psvals.Master_Bus_Reset:      [capvals.HW_Reset_Supported],
psvals.Diagnostic_Interrupt__NMI_:[capvals.HW_Reset_Supported],
psvals.Off___Soft_Graceful:       [capvals.Graceful_Shutdown_Supported],
psvals.Off___Hard_Graceful:       [capvals.Graceful_Shutdown_Supported],
psvals.Master_Bus_Reset_Graceful: [capvals.HW_Reset_Supported,
                                   capvals.Graceful_Shutdown_Supported],
psvals.Power_Cycle__Off___Soft_Graceful_: [capvals.Power_Cycling_Supported,
                                   capvals.Graceful_Shutdown_Supported],
psvals.Power_Cycle__Off___Hard_Graceful_:[capvals.Graceful_Shutdown_Supported,
                                    capvals.Off_Hard_Power_Cycling_Supported],
}

